﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.82
@EndOfDesignText@
Sub Class_Globals
	Private fx As JFX
	Private Grp As sgeGroup
	Private NinePatch As sgeNinePatch
	Private BtnHeight As Double
	Private ChoiceBtn As sgeActor
	Private Text As sgeText
	Private TextA As sgeActor
	Private List As clsList
	Private Choices As List
	Private CBStyle As String
	Private Pressed As Boolean
	Private IsOpen As Boolean
	Private UsingMouse As Boolean
	Private LastIndex As Int
	Private Module As Object
	Private Event As String

	Private Const FONT_SIZE As Int = 16
	Private Const LEFT_MARGIN_FOR_TEXT As Int = 3
End Sub

'Creates a list that is opened/closed by pressing a button. The chosen item is displayed on the button.
'Raises an event with the specified name when a new choice is made.
Public Sub Initialize(Parent As sgeGroup, LocalX As Double, LocalY As Double, Width As Double, ButtonHeight As Double, ListHeight As Double, Content As List, Style As String, EventModule As Object, EventName As String)
	Grp.Initialize(Parent, "")
	Grp.SetLocalPosition(LocalX, LocalY)
	Grp.Tag = Me
	BtnHeight = ButtonHeight
	List.Initialize(Parent, LocalX, LocalY + ButtonHeight - 1, Width, ListHeight, Style, Me, "Choice_Click")
	For i = 0 To Content.Size - 1
		List.AddText(Content.Get(i))
	Next
	CBStyle = Style
	Pressed = False
	Load9Patch
	ChoiceBtn.InitializeWithNinePatch(NinePatch, Grp, "")
	ChoiceBtn.Width = Width
	Text.Initialize("", fx.DefaultFont(FONT_SIZE))
	TextA.InitializeWithText(Text, Grp, "")
	Choices = Content
	ChangeStyle(CBStyle)
	IsOpen = False
	UsingMouse = False
	LastIndex = -1
	Module = EventModule
	Event = EventName
End Sub

Private Sub Load9Patch
	'Loads the 9-patch image for the button
	If Pressed Then
		NinePatch.Initialize(UIManager.AM.SplitNinePatch(CBStyle & "_ChoiceButtonPressed"))
	Else
		NinePatch.Initialize(UIManager.AM.SplitNinePatch(CBStyle & "_ChoiceButton"))
	End If
End Sub

Private Sub UpdateUI
	'Loads the 9-patch image for the button
	Load9Patch

	'Adjusts the actors according to the 9-patch size
	ChoiceBtn.Height = Max(BtnHeight, Max(TextA.Height, NinePatch.MinHeight))
	List.SetTop(Grp.Y + ChoiceBtn.Height - 1)
	TextA.LocalX = NinePatch.FirstColumnWidth + LEFT_MARGIN_FOR_TEXT
	TextA.LocalY = (ChoiceBtn.Height - TextA.Height) / 2 - FONT_SIZE / 16

	'The list is only visible when opened
	List.SetVisible(IsOpen)

	'Ensures that the list is above all other objects
	If IsOpen Then
		List.SetZOrder(Grp.Parent.Children.Size)
	End If
End Sub

Public Sub ChangeStyle(NewStyle As String)
	CBStyle = NewStyle
	UpdateUI
	List.ChangeStyle(NewStyle)
End Sub

Public Sub RemoveFocus
	'Closes the list
	IsOpen = False
	UpdateUI
End Sub

Private Sub OpenList
	IsOpen = True
	UpdateUI
End Sub

Private Sub CloseList
	IsOpen = False
	UpdateUI
End Sub

Public Sub KeyPressed(KeyData As sgeKeyEvent)
	If IsOpen Then
		If KeyData.Code = KeyData.KEY_ENTER Or KeyData.Code = KeyData.KEY_ESCAPE _
		Or KeyData.Code = KeyData.KEY_PAGE_UP Or KeyData.Code = KeyData.KEY_SPACE Then
			'Enter/Escape/PgUp/Space close the list
			Pressed = True
			CloseList
		Else
			'Any other key -> delegates the event
			UsingMouse = False
			List.KeyPressed(KeyData)
		End If
	Else If KeyData.Code = KeyData.KEY_ENTER Or KeyData.Code = KeyData.KEY_PAGE_DOWN _
		 Or KeyData.Code = KeyData.KEY_SPACE Or KeyData.IsArrowKey Then
		'Enter/PgDown/Space/Arrows open the list
		Pressed = True
		OpenList
	End If
End Sub

Public Sub KeyReleased(KeyData As sgeKeyEvent)
	Pressed = False
	UpdateUI
End Sub

Public Sub Button_MousePressed(EventData As MouseEvent)
	'Opens/closes the list
	Pressed = True
	If IsOpen Then
		CloseList
	Else
		OpenList
	End If
End Sub

Public Sub List_MousePressed(EventData As MouseEvent)
	'Delegates the event to the list
	UsingMouse = True
	List.MousePressed(EventData)
End Sub

Public Sub MouseReleased(EventData As MouseEvent)
	Pressed = False
	UpdateUI
End Sub

Public Sub Choice_Click(Index As Int)
	'Displays the chosen value and raises an event
	If LastIndex <> Index Then
		LastIndex = Index
		Text.Text = Choices.Get(Index)
		TextA.SetSize(Text.TextWidth, Text.TextHeight)
		CallSub2(Module, Event, Index)
	End If

	'Closes the list if the item was selected with the mouse
	If UsingMouse Then
		Sleep(100) 'Lets some time to display the chosen item in the list
		CloseList
	End If
End Sub
