﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.82
@EndOfDesignText@
Sub Class_Globals
	Private fx As JFX
	Private Grp As sgeGroup
	Private GrpText As sgeGroup
	Private BkgdImg As sgeActor
	Private Bkgd_9Patch As sgeNinePatch
	Private CursorImg As sgeActor
	Private Cursor_9Patch As sgeNinePatch
	Private SelectionImg As sgeActor
	Private Text, CompuText As sgeText
	Private TextA As sgeActor
	Private CursorPosition As Int
	Private SelectionStart As Int
	Private Dragging As Boolean
	Private EditStyle As String
	Private Module As Object
	Private Event As String
	Private IsFocused As Boolean
	Type typHistory(PreviousPosition As Int, PreviousText As String)
	Private History As List

	Private Const FONT_SIZE As Int = 16
	Private Const BLINKING_TIME As Int = 400
	Private Const LEFT_MARGIN As Int = 1
	Private Const HISTORY_SIZE As Int = 50
End Sub

'Creates a text field to edit a string. It does not support multi-lines, tabs or right alignment.
'Raises an event with the specified name when the text is changed.
Public Sub Initialize(Parent As sgeGroup, LocalX As Double, LocalY As Double, Width As Double, Height As Double, Style As String, EventModule As Object, EventName As String)
	Grp.Initialize(Parent, "")
	Grp.SetLocalPosition(LocalX, LocalY)
	Grp.Tag = Me
	IsFocused = False
	EditStyle = Style
	LoadAssets
	BkgdImg.InitializeWithNinePatch(Bkgd_9Patch, Grp, "")
	BkgdImg.SetSize(Width, Height)
	GrpText.Initialize(Grp, "")
	GrpText.Tag = Me
	SelectionImg.InitializeWithImage(UIManager.AM.GetImage(EditStyle & "_Selected"), GrpText, "")
	Text.Initialize("", fx.DefaultFont(FONT_SIZE))
	TextA.InitializeWithText(Text, GrpText, "")
	CompuText.Initialize("", Text.Font)
	CursorImg.InitializeWithNinePatch(Cursor_9Patch, Grp, "")
	Module = EventModule
	Event = EventName
	CursorPosition = 0
	History.Initialize
	SelectionStart = 0
	Dragging = False
	EnableCursor(False)
	UpdateUI
End Sub

Private Sub LoadAssets
	'Loads the 9-patch images
	If IsFocused Then
		Bkgd_9Patch.Initialize(UIManager.AM.SplitNinePatch(EditStyle & "_EditText_Focused"))
	Else
		Bkgd_9Patch.Initialize(UIManager.AM.SplitNinePatch(EditStyle & "_EditText_Default"))
	End If
	Cursor_9Patch.Initialize(UIManager.AM.SplitNinePatch(EditStyle & "_EditText_Cursor"))

	'Loads the image for the selection rectangle
	If SelectionImg.IsInitialized Then
		SelectionImg.SetImage(UIManager.AM.GetImage(EditStyle & "_Selected"))
	End If
End Sub

Private Sub UpdateUI
	'Computes the position of the text according to the 9-patch used for the background
	GrpText.LocalX = Bkgd_9Patch.FirstColumnWidth + LEFT_MARGIN
	GrpText.LocalY = (BkgdImg.Height + Bkgd_9Patch.FirstRowHeight - Bkgd_9Patch.LastRowHeight - TextA.Height) / 2 - 1

	'Computes the size of the cursor
	Dim MaxHeight As Int = BkgdImg.Height - Bkgd_9Patch.FirstRowHeight - Bkgd_9Patch.LastRowHeight + 2
	CursorImg.Width = Cursor_9Patch.MinWidth
	CursorImg.Height = Min(MaxHeight, TextA.Height + 2)
	CursorImg.LocalY = GrpText.LocalY + (TextA.Height - CursorImg.Height) / 2 + 1

	'The selection rectangle has the same height than the cursor
	SelectionImg.Height = CursorImg.Height
	SelectionImg.Y = CursorImg.Y

	'Defines the bounds of the clipping rectangle for the text
	Dim MaxWidth As Int = BkgdImg.Width - Bkgd_9Patch.LastColumnWidth - Bkgd_9Patch.FirstColumnWidth - LEFT_MARGIN + 1
	GrpText.SetClipBounds(0, 0, MaxWidth, GrpText.Height)

	'Computes the position of the cursor and scrolls the text if necessary
	PlaceCursor
End Sub

Public Sub ChangeStyle(NewStyle As String)
	EditStyle = NewStyle
	LoadAssets
	UpdateUI
End Sub

Private Sub EnableCursor(Enabled As Boolean)
	CursorImg.Visible = False
	If Enabled Then
		Dim Actions As sgeActions
		Dim Const Blinking As sgeAction = Actions.ChangeVisibility.Then(Actions.Wait(BLINKING_TIME))
		CursorImg.SetRepeatedAction(Blinking, CursorImg.REPEAT_FOREVER)
	Else
		CursorImg.ClearActions
	End If
End Sub

Private Sub PlaceCursor
	'Computes the position of the cursor and scrolls the text if necessary
	CompuText.Text = Text.Text.SubString2(0, CursorPosition)
	Dim Position As Double = CompuText.TextWidth
	Dim MaxWidth As Int = BkgdImg.Width - Bkgd_9Patch.LastColumnWidth - Bkgd_9Patch.FirstColumnWidth - LEFT_MARGIN
	If Position > MaxWidth - TextA.LocalX Then
		'The text has to scroll so the cursor can move forward
		TextA.LocalX = MaxWidth - Position
	Else If Position < -TextA.LocalX Then
		'The text has to scroll so the cursor can move backward
		TextA.LocalX = -Position
	End If
	If TextA.LocalX + Text.TextWidth < MaxWidth Then
		'Fills the available space with the text when the text has scrolled and some characters were removed
		TextA.LocalX = Min(0, MaxWidth - Text.TextWidth)
	End If
	CursorImg.LocalX = Floor(Position + TextA.LocalX + GrpText.LocalX - 1)

	'The selection rectangle is visible if it does not start and end where the cursor is
	If SelectionStart = CursorPosition Then
		SelectionImg.Visible = False
	Else
		CompuText.Text = Text.Text.SubString2(0, SelectionStart)
		Dim SelPosition As Double = CompuText.TextWidth
		If CursorPosition > SelectionStart Then
			'The selection rectangle starts before the cursor
			SelectionImg.LocalX = SelPosition + TextA.LocalX
		Else
			'The selection rectangle ends after the cursor
			SelectionImg.LocalX = Position + TextA.LocalX
		End If
		SelectionImg.Width = Abs(Position - SelPosition)
		SelectionImg.Visible = True
	End If
End Sub

Private Sub MoveCursorTo(Position As Int, Selection As Boolean)
	CursorPosition = Max(0, Min(Position, Text.Text.Length))
	If Not(Selection) Then
		SelectionStart = CursorPosition
	End If
	PlaceCursor
End Sub

Public Sub GetCursorPosition As Int
	Return CursorPosition
End Sub

Public Sub SetCursorPosition(Position As Int)
	MoveCursorTo(CursorPosition, False)
End Sub

Public Sub Activate(Focused As Boolean)
	IsFocused = Focused
	LoadAssets
	EnableCursor(IsFocused)
End Sub

Public Sub GetSelectionLength As Int
	Return Abs(CursorPosition - SelectionStart)
End Sub

Public Sub GetSelectionStart As Int
	Return SelectionStart
End Sub

Private Sub UpdateText(NewText As String)
	Text.Text = NewText
	TextA.SetSize(Text.TextWidth, Text.TextHeight)
End Sub

Public Sub GetText As String
	Return Text.Text
End Sub

Public Sub SetText(NewText As String)
	UpdateText(NewText)
	MoveCursorTo(CursorPosition, False)
End Sub

Private Sub UpdateAndRaiseEvent(NewText As String, NewPosition As Int)
	UpdateText(NewText)
	MoveCursorTo(NewPosition, False)
	CallSub2(Module, Event, NewText)
End Sub

Private Sub RemoveSelection
	'Deletes the selected text
	If CursorPosition > SelectionStart Then
		Dim NewPosition As Int = CursorPosition - GetSelectionLength
		UpdateAndRaiseEvent(Text.Text.SubString2(0, NewPosition) & Text.Text.SubString(CursorPosition), NewPosition)
	Else
		UpdateAndRaiseEvent(Text.Text.SubString2(0, CursorPosition) & Text.Text.SubString(CursorPosition + GetSelectionLength), CursorPosition)
	End If
End Sub

Private Sub ReplaceSelectionWithText(Txt As String)
	'Replaces the current selection with the given text or adds the text if nothing is selected
	If CursorPosition <> SelectionStart Then
		RemoveSelection
	End If
	UpdateAndRaiseEvent(Text.Text.SubString2(0, CursorPosition) & Txt & Text.Text.SubString(CursorPosition), CursorPosition + Txt.Length)
End Sub

Private Sub AddToHistory
	'Stores in the history the current text and cursor position
	Dim H As typHistory
	H.Initialize
	H.PreviousPosition = CursorPosition
	H.PreviousText = Text.Text
	History.Add(H)

	'Limits the history size
	If History.Size > HISTORY_SIZE Then
		History.RemoveAt(0)
	End If
End Sub

Public Sub KeyPressed(KeyData As sgeKeyEvent)
	'Handles the special keys (arrows, home, delete, etc.)
	CursorImg.Visible = True
	Dim Input As sgeInput = UIManager.SGE.Input
	If KeyData.Code = KeyData.KEY_LEFT Then
		'Moves the cursor to the left
		MoveCursorTo(CursorPosition - 1, Input.IsShiftKeyDown)
	Else If KeyData.Code = KeyData.KEY_RIGHT Then
		'Moves the cursor to the right
		MoveCursorTo(CursorPosition + 1, Input.IsShiftKeyDown)
	Else If KeyData.Code = KeyData.KEY_HOME Then
		'Moves the cursor to the start
		MoveCursorTo(0, Input.IsShiftKeyDown)
	Else If KeyData.Code = KeyData.KEY_END Then
		'Moves the cursor to the end
		MoveCursorTo(Text.Text.Length, Input.IsShiftKeyDown)
	Else If KeyData.Code = KeyData.KEY_BACK_SPACE Then
		If CursorPosition = SelectionStart Then
			'Deletes the character before the cursor
			If CursorPosition > 0 Then
				AddToHistory
				UpdateAndRaiseEvent(Text.Text.SubString2(0, CursorPosition - 1) & Text.Text.SubString(CursorPosition), CursorPosition - 1)
			End If
		Else 'Deletes the selected text
			AddToHistory
			RemoveSelection
		End If
	Else If KeyData.Code = KeyData.KEY_DELETE Then
		If CursorPosition = SelectionStart Then
			'Deletes the character after the cursor
			If CursorPosition < Text.Text.Length Then
				AddToHistory
				UpdateAndRaiseEvent(Text.Text.SubString2(0, CursorPosition) & Text.Text.SubString(CursorPosition + 1), CursorPosition)
			End If
		Else 'Deletes the selected text
			AddToHistory
			RemoveSelection
		End If
	Else If KeyData.Code = "C" And Input.IsControlKeyDown Then 'CTRL+C
		'Copies the selected text to the clipboard
		If CursorPosition < SelectionStart Then
			fx.Clipboard.SetString(Text.Text.SubString2(CursorPosition, SelectionStart))
		Else If CursorPosition > SelectionStart Then
			fx.Clipboard.SetString(Text.Text.SubString2(SelectionStart, CursorPosition))
		End If
	Else If KeyData.Code = "V" And Input.IsControlKeyDown Then 'CTRL+V
		'Pastes the clipboard content (removes Tab, CR and LF from the pasted text)
		If fx.Clipboard.HasString Then
			AddToHistory
			ReplaceSelectionWithText(fx.Clipboard.GetString.Replace(Chr(9), " ").Replace(Chr(10), "").Replace(Chr(13), ""))
		End If
	Else If KeyData.Code = "X" And Input.IsControlKeyDown Then 'CTRL+X
		'Copies the selected text to the clipboard, then deletes it
		If CursorPosition <> SelectionStart Then
			AddToHistory
			If CursorPosition < SelectionStart Then
				fx.Clipboard.SetString(Text.Text.SubString2(CursorPosition, SelectionStart))
			Else If CursorPosition > SelectionStart Then
				fx.Clipboard.SetString(Text.Text.SubString2(SelectionStart, CursorPosition))
			End If
			RemoveSelection
		End If
	Else If KeyData.Code = "Z" And Input.IsControlKeyDown Then 'CTRL+Z
		'Reverts to the text stored in the history
		If History.Size > 0 Then
			Dim H As typHistory = History.Get(History.Size - 1)
			UpdateAndRaiseEvent(H.PreviousText, H.PreviousPosition)
			History.RemoveAt(History.Size - 1)
		End If
	End If
End Sub

Public Sub KeyTyped(UnicodeCharacter As String)
	'Handles the normal keys (a..z, 0..9, etc.)
	Dim Ascii As Int = Asc(UnicodeCharacter)
	Dim Input As sgeInput = UIManager.SGE.Input
	If Ascii <> Input.ASCII_BACK_SPACE And Ascii <> Input.ASCII_DELETE And Ascii <> Input.ASCII_ENTER _
	And Ascii <> Input.ASCII_ESCAPE And Ascii <> Input.ASCII_TAB And Not(Input.IsControlKeyDown) Then
		AddToHistory
		ReplaceSelectionWithText(UnicodeCharacter)
	End If
End Sub

Private Sub FollowTheMouse(EventData As MouseEvent)
	'Places the cursor where the mouse is
	Dim MousePos As sgePoint2D = UIManager.SGE.Input.MouseToCanvas(EventData.X, EventData.Y)
	MousePos = MousePos.Subtract(TextA.X, 0)
	For i = 1 To Text.Text.Length
		CompuText.Text = Text.Text.SubString2(0, i)
		If CompuText.TextWidth >= MousePos.X Then
			MoveCursorTo(i - 1, Dragging)
			Return
		End If
	Next

	'The mouse position is after the last character
	MoveCursorTo(Text.Text.Length, Dragging)
End Sub

Public Sub MousePressed(EventData As MouseEvent)
	FollowTheMouse(EventData)
	Dragging = True
End Sub

Public Sub MouseDragged(EventData As MouseEvent)
	If Dragging Then
		FollowTheMouse(EventData)
	End If
End Sub

Public Sub MouseReleased(EventData As MouseEvent)
	Dragging = False
End Sub

Public Sub PerformAction(ElapsedSinceLastFrame As Double)
	'Performs the blinking action if focused
	If IsFocused Then
		CursorImg.PerformAction(ElapsedSinceLastFrame)
	End If
End Sub
