﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=5.82
@EndOfDesignText@
Sub Class_Globals
	Private fx As JFX
	Private Grp As sgeGroup
	Private GrpItems As sgeGroup
	Private GrpVSBar As sgeGroup
	Private ListStyle As String
	Private MinClipY As Double
	Private MaxClipY As Double
	Private ClipHeight As Double
	Private BkgdImg As sgeActor
	Private Bkgd_9Patch As sgeNinePatch
	Private ScrollbarImg As sgeActor
	Private Scrollbar_9Patch As sgeNinePatch
	Private ScrollKnobImg As sgeActor
	Private ScrollKnob_9Patch As sgeNinePatch
	Private SelectedImg As sgeActor
	Private Module As Object
	Private Event As String
	Private ItemsHeight As Double
	Private ItemsCount As Int
	Private SelectedIndex As Int
	Private ScrollPercent As Double
	Private DraggingKnob As Boolean

	Private Const MIN_SCROLLBAR_WIDTH As Int = 18
	Private Const MIN_KNOB_HEIGHT As Int = 20
	Private Const FONT_SIZE As Int = 16
	Private Const LEFT_MARGIN_FOR_TEXT As Int = 4
End Sub

'Creates a vertical list.
'Raises an event with the specified name when an item is selected.
Public Sub Initialize(Parent As sgeGroup, LocalX As Double, LocalY As Double, Width As Double, Height As Double, Style As String, EventModule As Object, EventName As String)
	Grp.Initialize(Parent, "")
	Grp.SetLocalPosition(LocalX, LocalY)
	Grp.Tag = Me
	ListStyle = Style
	LoadAssets
	BkgdImg.InitializeWithNinePatch(Bkgd_9Patch, Grp, "")
	BkgdImg.SetSize(Width, Height)
	GrpItems.Initialize(Grp, "")
	GrpItems.SetLocalPosition(1, 1)
	GrpItems.Tag = Me
	MinClipY = GrpItems.Y
	MaxClipY = GrpItems.Y + Height - 2
	ClipHeight = MaxClipY - MinClipY + 1
	SelectedImg.InitializeWithImage(UIManager.AM.GetImage(Style & "_Selected"), GrpItems, "")
	GrpVSBar.Initialize(Grp, "")
	GrpVSBar.LocalY = 1
	GrpVSBar.Tag = Me
	ScrollbarImg.InitializeWithNinePatch(Scrollbar_9Patch, GrpVSBar, "")
	ScrollbarImg.Height = Height - 2
	ScrollKnobImg.InitializeWithNinePatch(ScrollKnob_9Patch, GrpVSBar, "")
	ScrollKnobImg.LocalX = 1
	UpdateUI
	Module = EventModule
	Event = EventName
	ItemsHeight = 0
	ItemsCount = 0
	SetSelectedIndex(-1)
	DraggingKnob = False
End Sub

Private Sub LoadAssets
	'Loads the 9-patch images
	Bkgd_9Patch.Initialize(UIManager.AM.SplitNinePatch(ListStyle & "_List"))
	Scrollbar_9Patch.Initialize(UIManager.AM.SplitNinePatch(ListStyle & "_Scrollbar"))
	ScrollKnob_9Patch.Initialize(UIManager.AM.SplitNinePatch(ListStyle & "_ScrollbarKnob"))

	'Loads the image for the selection rectangle
	If SelectedImg.IsInitialized Then
		SelectedImg.SetImage(UIManager.AM.GetImage(ListStyle & "_Selected"))
	End If
End Sub

Private Sub UpdateUI
	'Adjusts the size of actors
	ScrollbarImg.Width = Max(Scrollbar_9Patch.MinWidth, Max(ScrollKnob_9Patch.MinWidth + 2, MIN_SCROLLBAR_WIDTH))
	ScrollKnobImg.Height = Max(ScrollKnob_9Patch.MinHeight, MIN_KNOB_HEIGHT)
	ScrollKnobImg.Width = ScrollbarImg.Width - 2

	'The scrollbar is visible if the list is not tall enough to display all items
	GrpVSBar.Visible = (ItemsHeight > MaxClipY - MinClipY - 1)
	If GrpVSBar.Visible Then
		GrpVSBar.LocalX = BkgdImg.Width - ScrollbarImg.Width - 1
		SelectedImg.Width = GrpVSBar.LocalX + 2
	Else
		SelectedImg.Width = BkgdImg.Width - 2
	End If
	SetSelectedIndex(SelectedIndex)
End Sub

Public Sub ChangeStyle(NewStyle As String)
	ListStyle = NewStyle
	LoadAssets
	UpdateUI
End Sub

'Adds a text to the list.
Public Sub AddText(Text As String)
	Dim TextObj As sgeText
	TextObj.Initialize(Text, fx.DefaultFont(FONT_SIZE))
	Dim TextA As sgeActor
	TextA.InitializeWithText(TextObj, GrpItems, "")
	TextA.LocalX = LEFT_MARGIN_FOR_TEXT
	AddObject(TextA)
End Sub

'Adds an object (sgeActor or sgeGroup) to the list.
Public Sub AddObject(Obj As Object)
	Dim Item As sgeGroup
	If Obj Is sgeActor Then
		Item.Initialize(GrpItems, "")
		Dim A As sgeActor = Obj
		A.Parent = Item
	Else 'Obj is sgeGroup
		Item = Obj
		Item.Parent = GrpItems
	End If
	Item.LocalY = ItemsHeight
	Item.Tag = Me
	ItemsHeight = ItemsHeight + Item.Height
	ItemsCount = ItemsCount + 1
	UpdateUI
End Sub

Public Sub GetSelectedIndex As Int
	Return SelectedIndex
End Sub

Public Sub SetSelectedIndex(Index As Int)
	SelectedIndex = Max(-1, Min(Index, ItemsCount - 1))
	If SelectedIndex > -1 Then
		'Sets the height and vertical position of the selection rectangle
		Dim Item As sgeGroup = GrpItems.Children.Get(SelectedIndex + 1)
		SelectedImg.Height = Item.Height + 2
		SelectedImg.LocalY = Item.LocalY

		'Adjusts the position of the items group if the selection is outside the visible range
		If SelectedImg.Y < MinClipY Then
			GrpItems.Y = GrpItems.Y - SelectedImg.Y + MinClipY
		Else If SelectedImg.Y + SelectedImg.Height > MaxClipY Then
			GrpItems.Y = GrpItems.Y - SelectedImg.Y - SelectedImg.Height + MaxClipY
		End If

		'The selection rectangle is visible when an item is selected
		SelectedImg.Visible = True
	Else
		'The selection rectangle is invisible when no item is selected
		SelectedImg.Visible = False
	End If

	'Computes the new vertical scroll percentage
	ScrollPercent = GrpItems.LocalY / (ClipHeight - ItemsHeight - 2)

	'Defines the bounds of the clipping rectangle
	Clip

	'Sets the vertical position of the scrollbar knob
	ComputeKnobPosition
End Sub

Private Sub Clip
	'Defines the bounds of the clipping rectangle
	GrpItems.SetClipBounds(0, 1 - GrpItems.LocalY, SelectedImg.Width, ClipHeight)
End Sub

Public Sub KeyPressed(KeyData As sgeKeyEvent)
	'Down/Up/Home/End: changes the selected item
	If KeyData.Code = KeyData.KEY_DOWN Then
		If SelectedIndex < ItemsCount - 1 Then
			SetSelectedIndex(SelectedIndex + 1)
			CallSub2(Module, Event, SelectedIndex)
		End If
	Else If KeyData.Code = KeyData.KEY_UP Then
		If SelectedIndex > 0 Then
			SetSelectedIndex(SelectedIndex - 1)
			CallSub2(Module, Event, SelectedIndex)
		End If
	Else If KeyData.Code = KeyData.KEY_HOME Then
		If SelectedIndex <> 0 Then
			SetSelectedIndex(0)
			CallSub2(Module, Event, SelectedIndex)
		End If
	Else If KeyData.Code = KeyData.KEY_END Then
		If SelectedIndex <> ItemsCount - 1 Then
			SetSelectedIndex(ItemsCount - 1)
			CallSub2(Module, Event, SelectedIndex)
		End If
	End If
End Sub

Private Sub ComputeKnobPosition
	'Sets the vertical position of the scrollbar knob
	Dim ScrollRange As Double = ScrollbarImg.Height - ScrollKnobImg.Height - 2
	ScrollKnobImg.LocalY = Round(ScrollRange * ScrollPercent) + 1
End Sub

Private Sub FollowTheMouse(MousePos As sgePoint2D)
	'Computes the new vertical scroll percentage according to the mouse position
	Dim RelativePos As Double = MousePos.Y - ScrollbarImg.Y - (ScrollKnobImg.Height / 2)
	ScrollPercent = Max(0, Min(RelativePos / (ScrollbarImg.Height - ScrollKnobImg.Height), 1))

	'Sets the vertical position of the items group
	GrpItems.LocalY = ScrollPercent * (ClipHeight - ItemsHeight - 2)

	'Defines the bounds of the clipping rectangle
	Clip

	'Sets the vertical position of the scrollbar knob
	ComputeKnobPosition
End Sub

Private Sub HandleMousePressedEvent(MousePos As sgePoint2D)
	'Handles the Pressed event
	Dim PressedList As List = Grp.FindAtPos2(MousePos, Grp.FIND_ONLY_GROUPS, True, False)
	If PressedList.Size > 0 Then
		If PressedList.Get(0) = GrpVSBar Then
			'The mouse is over the scrollbar
			PressedList = GrpVSBar.FindAtPos2(MousePos, Grp.FIND_ONLY_ACTORS, True, False)
			If PressedList.Size > 0 Then
				If PressedList.Get(0) = ScrollbarImg Then
					'Not on the knob -> sets the new position of the items group and scrollbar knob
					FollowTheMouse(MousePos)
				Else If PressedList.Get(0) = ScrollKnobImg Then
					'On the knob -> waits to be dragged
					DraggingKnob = True
				End If
			End If
		Else If PressedList.Get(0) = GrpItems Then
			'The mouse is outside an item but over the items group, so a new check is done with LocalX = 0
			MousePos.Initialize(GrpItems.X, MousePos.Y)
			HandleMousePressedEvent(MousePos)
		Else
			'The mouse is over an item
			For i = 0 To GrpItems.Children.Size - 1
				Dim O As Object = GrpItems.Children.Get(i)
				If O Is sgeGroup Then
					Dim G As sgeGroup = O
					If G = PressedList.Get(0) Then
						SetSelectedIndex(i - 1)
						CallSub2(Module, Event, SelectedIndex)
						Return
					End If
				End If
			Next
		End If
	End If
End Sub

Public Sub MousePressed(EventData As MouseEvent)
	Dim MousePos As sgePoint2D = UIManager.SGE.Input.MouseToCanvas(EventData.X, EventData.Y)
	HandleMousePressedEvent(MousePos)
End Sub

Public Sub MouseDragged(EventData As MouseEvent)
	If DraggingKnob Then
		'Sets the new position of the items group and scrollbar knob
		FollowTheMouse(UIManager.SGE.Input.MouseToCanvas(EventData.X, EventData.Y))
	End If
End Sub

Public Sub MouseReleased(EventData As MouseEvent)
	DraggingKnob = False
End Sub

Public Sub Scroll(DeltaY As Double)
	If GrpVSBar.Visible Then
		'Updates the vertical scroll percentage
		Dim ScrollRange As Double = ScrollbarImg.Height - 2
		ScrollPercent = Max(0, Min(ScrollPercent - (ScrollRange / DeltaY / 100), 1))

		'Sets the vertical position of the items group
		GrpItems.LocalY = ScrollPercent * (ClipHeight - ItemsHeight - 2)

		'Defines the bounds of the clipping rectangle
		Clip

		'Sets the vertical position of the scrollbar knob
		ComputeKnobPosition
	End If
End Sub

Public Sub SetTop(NewTop As Double)
	Grp.Y = NewTop
	MinClipY = Grp.Y + 1
	MaxClipY = Grp.Y + BkgdImg.Height - 1
End Sub

Public Sub SetVisible(Visible As Boolean)
	Grp.Visible = Visible
End Sub

Public Sub SetZOrder(ZOrder As Int)
	Grp.ZOrder = ZOrder
End Sub
